package se.plweb.memory.domain;

/**
 * @author Peter Lindblom
 * 
 */

public class GameObjectImpl implements GameObject {

	private static final long serialVersionUID = 1L;

	private int value;
	private Position position = new Position();
	private GameObjectState state;

	public GameObjectImpl(int value, Position position) {
		this.value = value;
		this.position = position;
		setState(GameObjectState.NORMAL_STATE);
	}

	public synchronized int getValue() {
		return value;
	}

	public synchronized GameObjectState getState() {
		return state;
	}

	public synchronized void setState(GameObjectState value) {
		state = value;
	}

	public synchronized void setValue(int value) {
		this.value = value;
	}

	public synchronized boolean isInNormalState() {
		if (getState() != GameObjectState.MATCHED_STATE
				&& this.getState() != GameObjectState.PRESSED_STATE
				&& this.getState() != GameObjectState.DISABLED_STATE) {
			return true;
		}
		return false;
	}

	public synchronized boolean hasTheSameValueAndNotTheSameCoordinates(GameObject gameObject) {
		if (gameObject.getValue() == this.getValue()
				&& !this.position.equals(gameObject.getPosition())) {
			return true;
		} else {
			return false;
		}
	}

	public synchronized boolean hasTheSameValueAndTheSameCoordinates(GameObject gameObject) {
		if (gameObject.getValue() == this.getValue()
				&& gameObject.getPosition().equals(this.position)) {
			return true;
		} else {
			return false;
		}
	}

	@Override
	public String toString() {
		return "GameObjectImpl value=" + getValue() + ", state=" + getState()
				+ getPosition().toString();
	}

	@Override
	public synchronized boolean equals(Object obj) {
		if (!(obj instanceof GameObject)) {
			return false;
		} else {
			return hasTheSameValueAndTheSameCoordinates((GameObject) obj);
		}
	}

	@Override
	public int hashCode() {
		int tmpHash = 0;
		tmpHash += position.hashCode();
		tmpHash += getState().hashCode();
		tmpHash += getValue();
		return tmpHash;
	}

	public synchronized Position getPosition() {
		return position.clone();
	}

	public synchronized void setPosition(Position position) {
		this.position = position;
	}

}
